;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Definitions for Dr. Scheme
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; Semi-standard functions and values

(define nil '())
(define (fold-right op init lst)
  (if (null? lst)
      init
      (op (car lst)
          (fold-right op init (cdr lst)))))
(define (filter pred lst)
  (cond ((null? lst) nil)
        ((pred (car lst)) 
         (cons (car lst)
               (filter pred (cdr lst))))
        (else (filter pred (cdr lst)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Error checking helpers
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (assert test-val pred)
  (if (pred test-val)
      #t
      (error "ASSERT FAILED: " 
             test-val pred)))
(define (assert-equal test-val expected-val)
  (assert test-val 
          (lambda (t) 
            (equal? t expected-val))))
(define (assert-f test-val)
  (assert-equal test-val #f))
(define (assert-t test-val)
  (assert-equal test-val #t))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Boolean expressions abstraction
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; Determines whether a given object is
;; a boolean expression
(define (expression? exp)
  (or (variable? exp) (not? exp)
      (or? exp)       (and? exp)))

;; Tells whether the given expression is
;; a simple variable
(define (variable? exp) 
  (and (pair? exp) (eq? (car exp) 'var)))
;; Turns a symbol into a variable for a
;; boolean expression
(define (make-variable sym) 
  (assert sym symbol?)
  (list 'var sym))
;; Returns the symbol associated with
;; the variable.  
(define (variable-name exp) 
  (assert exp variable?)
  (cadr exp))

;; Tells whether the given expression is
;; a not expression
(define (not? exp) 
  (and (pair? exp) (eq? (car exp) 'not)))
;; Creates a not expression from a 
;; subexpression
(define (make-not exp)
  (assert exp expression?) 
  (list 'not exp))
;; Returns the subexpression associated 
;; with a not expression
(define (not-operand exp) 
  (assert exp not?)
  (cadr exp))

;; Tells whether a given expression is
;; an or expression.
(define (or? exp)
  (and (pair? exp) (eq? (car exp) 'or)))
;; Creates an or expression of two 
;; subexpressions
(define (make-or exp1 exp2) 
  (assert exp1 expression?)
  (assert exp1 expression?)
  (list 'or exp1 exp2))
;; Returns the first subexpression of an
;; or expression
(define (or-first exp) 
  (assert exp or?)
  (cadr exp))
;; Returns the second subexpression of an
;; or expression
(define (or-second exp)
  (assert exp or?)
  (caddr exp))

;; Tells whether a given expression is
;; an and expression.
(define (and? exp)
  (and (pair? exp) (eq? (car exp) 'and)))
;; Creates an and expression of two 
;; subexpressions
(define (make-and exp1 exp2) 
  (assert exp1 expression?)
  (assert exp1 expression?)
  (list 'and exp1 exp2))
;; Returns the first subexpression of an
;; and expression
(define (and-first exp) 
  (assert exp and?)
  (cadr exp))
;; Returns the second subexpression of an
;; and expression
(define (and-second exp)
  (assert exp and?)
  (caddr exp))

