;; Returns the value of the boolean expression exp given
;; the variable bindings.
(define (formula-value bindings exp)
  (cond ((variable? exp)
         (variable-value bindings (variable-name exp)))
        ((not? exp)
         (not (formula-value bindings (not-operand exp))))
        ((or? exp)
         (or (formula-value bindings (or-first exp))
             (formula-value bindings (or-second exp))))
        ((and? exp)
         (and (formula-value bindings (and-first exp))
              (formula-value bindings (and-second exp))))
        (else (error "unknown exp" exp))))

;; Looks up the variable's value in a set of bindings.
;; Note that a better way to do this would be to use
;; an associative data structure to store the bindings,
;; but we haven't seen them yet.
(define (variable-value bindings varname)
  (set-check bindings)
  (assert-equal (length (filter (lambda (x) (eq? (first x) varname)) 
                                (set-to-list bindings)))
                1)
  (fold-right (lambda (val acc) (if (eq? (first val) varname)
                                    (second val)
                                    acc))
              nil
              (set-to-list bindings)))

; Tests

(define va (make-variable 'a))
(define vb (make-variable 'b))
(define vc (make-variable 'c))

(define e1 va)
(define e2 (make-not vb))
(define e3 (make-and va vb))
(define e4 (make-or va vb))
(define e5 (make-and (make-not va) (make-not vc)))
(define e6 (make-and (make-not va) va))
(define e7 (make-and (make-or va (make-not vb)) (make-or (make-not va) vc)))

(define b (list-to-set '((a #t) (b #f) (c #f))))

(assert-t (formula-value b e1))
(assert-t (formula-value b e2))
(assert-f (formula-value b e3))
(assert-t (formula-value b e4))
(assert-f (formula-value b e5))
(assert-f (formula-value b e6))
(assert-f (formula-value b e7))
