;; Returns the set of variables used in the given expression.
(define (formula-variables exp)
 (cond ((variable? exp)
        (set-add (variable-name exp) (new-set)))
       ((not? exp)
        (formula-variables (not-operand exp)))
       ((or? exp)
        (set-union (formula-variables (or-first exp))
                   (formula-variables (or-second exp))))
       ((and? exp)
        (set-union (formula-variables (and-first exp))
                   (formula-variables (and-second exp))))
       (else (error "unknown exp" exp))))

; Tests
(define va (make-variable 'a))
(define vb (make-variable 'b))
(define vc (make-variable 'c))

(define e1 va)
(define e2 (make-not vb))
(define e3 (make-and va vb))
(define e4 (make-or va vb))
(define e5 (make-and (make-not va) (make-not vc)))
(define e6 (make-and (make-not va) va))
(define e7 (make-and (make-or va (make-not vb)) (make-or (make-not va) vc)))

(assert-t (set-equal? (formula-variables e1) (list-to-set '(a))))
(assert-t (set-equal? (formula-variables e2) (list-to-set (list 'b))))
(assert-t (set-equal? (formula-variables e3) (list-to-set '(a b))))
(assert-t (set-equal? (formula-variables e4) (list-to-set '(b a))))
(assert-t (set-equal? (formula-variables e5) (list-to-set '(a c))))
(assert-t (set-equal? (formula-variables e6) (list-to-set '(a))))
(assert-t (set-equal? (formula-variables e7) (list-to-set '(a b c))))
