;; Returns #t if there is some assignment of boolean values to
;; each of the formula variables in exp such that the entire
;; boolean expression evaluates to true.
(define (sat exp)
  ; Recursively takes one variable out of unassigned-vars and
  ; tries both possible bindings (#t and #f).  bindings is a set
  ; of bindings, where a binding is a 2-element list whose first
  ; element is the variable names and whose second element is the
  ; bound boolean value.
  (define (assign-vars unassigned-vars bindings)
    (if (null? unassigned-vars)
        ; If all variables have been assigned, see if the expression
        ; returns #t
	(formula-value bindings exp)
	; If there are unassigned variables, try both possible assignments
	; of the first remaining variable.  Recurse.
	(or (assign-vars (cdr unassigned-vars)
			 (set-add (list (car unassigned-vars) #t) bindings))
	    (assign-vars (cdr unassigned-vars)
			 (set-add (list (car unassigned-vars) #f) bindings)))))
  (assign-vars (set-to-list (formula-variables exp)) (new-set)))

; tests
(define va (make-variable 'a))
(define vb (make-variable 'b))
(define vc (make-variable 'c))

(define e1 va)
(define e2 (make-not vb))
(define e3 (make-and va vb))
(define e4 (make-or va vb))
(define e5 (make-and (make-not va) (make-not vc)))
(define e6 (make-and (make-not va) va))
(define e7 (make-and (make-or va (make-not vb)) (make-or (make-not va) vc)))

(assert-t (sat e1))
(assert-t (sat e2))
(assert-t (sat e3))
(assert-t (sat e4))
(assert-t (sat e5))
(assert-f (sat e6))
(assert-t (sat e7))